function jsonToDataMatrix(json) {
    var matrix = [];
    var index = [];
    var columns = Object.keys(json[0]);
    json.forEach(function (row, i) {
        matrix.push(Object.values(row));
        index.push(i);
    });
    return { matrix: matrix, index: index, columns: columns };
}
/**
 * Adjust structure of data.
 *
 * To avoid missing keys and values.
 */
function adjustJSONData(json, mode) {
    if (mode === void 0) { mode = 'max'; }
    if (!json || json.length === 0)
        return [];
    var resultJSONData = [];
    // consistent fields
    var fieldSet = new Set();
    if (mode === 'max') {
        fieldSet = new Set();
        json.forEach(function (row) {
            Object.keys(row).forEach(function (key) {
                fieldSet.add(key);
            });
        });
    }
    else {
        // mode === 'min
        fieldSet = new Set(Object.keys(json[0]));
        json.forEach(function (row) {
            var keys = Object.keys(row);
            fieldSet.forEach(function (field) {
                if (!keys.includes(field)) {
                    fieldSet.delete(field);
                }
            });
        });
    }
    // adjust data row by row
    json.forEach(function (row) {
        var newRow = {};
        fieldSet.forEach(function (fieldName) {
            newRow[fieldName] = row[fieldName] || undefined;
        });
        resultJSONData.push(newRow);
    });
    return resultJSONData;
}
/**
 * @public
 */
var Dataset = /** @class */ (function () {
    function Dataset(type, params) {
        this.dataMatrix = [];
        this.dataIndex = [];
        this.dataColumns = [];
        switch (type) {
            case 'json': {
                var source = params.source;
                var data = adjustJSONData(source);
                var _a = jsonToDataMatrix(data), matrix = _a.matrix, index = _a.index, columns = _a.columns;
                this.dataMatrix = matrix;
                this.dataIndex = index;
                this.dataColumns = columns;
                break;
            }
            case 'rows':
                // TODO
                break;
            case 'columns':
                // TODO
                break;
            default:
                this.dataMatrix = [];
                this.dataIndex = [];
                this.dataColumns = [];
                break;
        }
    }
    Dataset.prototype.toJson = function () {
        var _this = this;
        return this.dataMatrix.map(function (row) {
            var rowObj = {};
            _this.dataColumns.forEach(function (key, i) {
                rowObj[key] = row[i];
            });
            return rowObj;
        });
    };
    return Dataset;
}());
export { Dataset };
